@@ -51,6 +51,7 @@ static void DestroyDino();
 static Boolean SetValuesDino();
 static void get_color();
 static void move_control_cb();
+static void check_polyhedrons();
 static void reset_polyhedrons();
 static void no_move_polyhedrons();
 static void practice_polyhedrons();
@@ -58,6 +59,8 @@ static void randomize_polyhedrons();
 static void move_polyhedrons();
 static void move_inside_corners();
 static void move_outside_corners();
+static void move_edges();
+static void move_faces();
 static void read_diagonal();
 static void write_diagonal();
 static void read_face();
@@ -148,6 +151,8 @@ static DinoLoc slide_corner[MAXFACES][MAXORIENT][MAXORIENT/2] =
   }
 };
 
+static int opp_face[MAXFACES] = {4, 3, 5, 1, 0, 2};
+
 static DinoCornerLoc opp_corner[MAXFACES][MAXORIENT] =
 {
   {{4, 3}, {4, 2}, {4, 1}, {4, 0}},
@@ -158,6 +163,8 @@ static DinoCornerLoc opp_corner[MAXFACES][MAXORIENT] =
   {{2, 3}, {2, 2}, {2, 1}, {2, 0}}
 };
 
+static int slide_next_face2[MAXFACES] = {4, 3, 5, 1, 0, 2};
+
 static DinoCornerLoc slide_next_face[MAXFACES][MAXORIENT] =
 {
   {{5, STRT}, {3,   CW}, {2, STRT}, {1,  CCW}},
@@ -168,6 +175,16 @@ static DinoCornerLoc slide_next_face[MAXFACES][MAXORIENT] =
   {{4, STRT}, {3, HALF}, {0, STRT}, {1, HALF}}
 };
 
+static int face_to_rotate2[MAXFACES][MAXORIENT][2] =
+{
+  {{3, 5}, {2, 3}, {1, 2}, {1, 5}},
+  {{0, 2}, {2, 4}, {4, 5}, {0, 5}},
+  {{3, 0}, {4, 3}, {1, 4}, {0, 1}},
+  {{0, 5}, {4, 5}, {2, 4}, {0, 2}},
+  {{2, 3}, {3, 5}, {1, 5}, {1, 2}},
+  {{4, 3}, {3, 0}, {0, 1}, {1, 4}}
+};
+
 static int face_to_rotate[MAXFACES][MAXORIENT] =
 {
   {3, 2, 1, 5},
@@ -186,6 +203,7 @@ Widget request, new;
   XtGCMask valueMask;
   int face;
 
+  check_polyhedrons(w);
   init_moves();
   reset_polyhedrons(w);
   w->dino.practice = FALSE;
@@ -255,8 +273,17 @@ Widget current, request, new;
     if (strcmp(w->dino.face_name[face], c->dino.face_name[face]))
       get_color(w, face, FALSE);
   }
-  if (w->dino.practice != c->dino.practice) {
+  if (w->dino.orient != c->dino.orient) {
     reset_polyhedrons(w);
+    w->dino.practice = FALSE;
+    redraw = TRUE;
+  } else if (w->dino.practice != c->dino.practice) {
+    reset_polyhedrons(w);
+    redraw = TRUE;
+  }
+  if (w->dino.mode != c->dino.mode) {
+    reset_polyhedrons(w);
+    w->dino.practice = FALSE;
     redraw = TRUE;
   }
   if (w->dino.face == DINO_RESTORE) {
@@ -326,7 +353,7 @@ int n_args;
 {
   FILE *fp;
   char c;
-  int orient, practice, moves;
+  int mode, orient, practice, moves;
   dinoCallbackStruct cb;
 
   if ((fp = fopen(DATAFILE, "r")) == NULL)
@@ -334,7 +361,30 @@ int n_args;
   else {
     flush_moves(w);
     while ((c = getc(fp)) != EOF && c != ':');
+    (void) fscanf(fp, "%d", &mode);
+    if (mode >= PERIOD2 && mode <= BOTH)
+      switch (mode) {
+        case PERIOD2:
+          cb.reason = DINO_PERIOD2;
+          XtCallCallbacks((Widget) w, XtNselectCallback, &cb);
+          break;
+        case PERIOD3:
+          cb.reason = DINO_PERIOD3;
+          XtCallCallbacks((Widget) w, XtNselectCallback, &cb);
+          break;
+        case BOTH:
+          cb.reason = DINO_BOTH;
+          XtCallCallbacks((Widget) w, XtNselectCallback, &cb);
+      }
+    else
+      (void) printf("%s corrupted: mode %d should be between %d and %d\n",
+         DATAFILE, mode, PERIOD2, BOTH);
+    while ((c = getc(fp)) != EOF && c != ':');
     (void) fscanf(fp, "%d", &orient);
+    if (w->dino.orient != (Boolean) orient) {
+      cb.reason = DINO_ORIENT;
+      XtCallCallbacks((Widget) w, XtNselectCallback, &cb);
+    }
     while ((c = getc(fp)) != EOF && c != ':');
     (void) fscanf(fp, "%d", &practice);
     if (w->dino.practice != (Boolean) practice) {
@@ -343,13 +393,13 @@ int n_args;
     }
     while ((c = getc(fp)) != EOF && c != ':');
     (void) fscanf(fp, "%d", &moves);
-    scan_start_position(fp);
+    scan_start_position(fp, w);
     cb.reason = DINO_RESTORE;
     XtCallCallbacks((Widget) w, XtNselectCallback, &cb);
     scan_moves(fp, w, moves);
     (void) fclose(fp);
-    (void) printf("%s: practice %d, moves %d.\n",
-      DATAFILE, practice, moves);
+    (void) printf("%s: mode %d, orient %d, practice %d, moves %d.\n",
+      DATAFILE, mode, orient, practice, moves);
   }
 }
 
@@ -364,9 +414,11 @@ int n_args;
   if ((fp = fopen(DATAFILE, "w")) == NULL)
     (void) printf("Can not write to %s.\n", DATAFILE);
   else {
+    (void) fprintf(fp, "dino: %d\n", w->dino.mode);
+    (void) fprintf(fp, "orient: %d\n", (w->dino.orient) ? 1 : 0);
     (void) fprintf(fp, "practice: %d\n", (w->dino.practice) ? 1 : 0);
     (void) fprintf(fp, "moves: %d\n", num_moves());
-    print_start_position(fp);
+    print_start_position(fp, w);
     print_moves(fp);
     (void) fclose(fp);
     (void) printf("Saved to %s.\n", DATAFILE);
@@ -410,29 +462,100 @@ int n_args;
   /* solve_polyhedrons(w); */ /* Sorry, unimplemented */
 }
 
-void move_dino_input(w, x, y, direction, shift, control)
+void orientize_dino(w, event, args, n_args)
 DinoWidget w;
-int x, y, direction, shift, control;
+XEvent *event;
+char *args[];
+int n_args;
 {
-  int face, side;
+  dinoCallbackStruct cb;
+ 
+  cb.reason = DINO_ORIENT;
+  XtCallCallbacks((Widget) w, XtNselectCallback, &cb);
+}
+
+void period2_mode_dino(w, event, args, n_args)
+DinoWidget w;
+XEvent *event;
+char *args[];
+int n_args;
+{
+  dinoCallbackStruct cb;
+ 
+  cb.reason = DINO_PERIOD2;
+  XtCallCallbacks((Widget) w, XtNselectCallback, &cb);
+}
+ 
+void period3_mode_dino(w, event, args, n_args)
+DinoWidget w;
+XEvent *event;
+char *args[];
+int n_args;
+{
+  dinoCallbackStruct cb;
+ 
+  cb.reason = DINO_PERIOD3;
+  XtCallCallbacks((Widget) w, XtNselectCallback, &cb);
+}
+ 
+void both_mode_dino(w, event, args, n_args)
+DinoWidget w;
+XEvent *event;
+char *args[];
+int n_args;
+{
+  dinoCallbackStruct cb;
+ 
+  cb.reason = DINO_BOTH;
+  XtCallCallbacks((Widget) w, XtNselectCallback, &cb);
+}
+ 
+void move_dino_input(w, x, y, direction, shift, control, alt)
+DinoWidget w;
+int x, y, direction, shift, control, alt;
+{
+  int face, side, style;
 
   if (!w->dino.practice && !control && check_solved(w)) {
     no_move_polyhedrons(w);
     return;
   }
-  if (w->dino.dim == 2 && !control && direction >= 2 * MAXORIENT)
-    return;
+  if (w->dino.dim == 2 && direction >= 2 * MAXORIENT) {
+    if (control)
+      style = FACE;
+    else
+       return;
+  } else if (w->dino.mode != BOTH) {
+    if (control && alt) {
+      if (w->dino.mode == PERIOD3)
+        style = EDGE;
+      else if (shift)
+        style = MIDDLE;
+      else
+        style = CORNER;
+    } else if (w->dino.mode == PERIOD2)
+      style = EDGE;
+    else if (shift)
+      style = MIDDLE;
+    else
+      style = CORNER;
+  } else {
+    if (alt)
+      style = EDGE;
+    else {
+      if (shift)
+        style = MIDDLE;
+      else
+        style = CORNER;
+    }
+  }
   if ((w->dino.dim == 2 && !position_polyhedrons2d((Dino2DWidget) w,
        x, y, &face, &side, &direction)) ||
       (w->dino.dim == 3 && !position_polyhedrons3d((Dino3DWidget) w,
        x, y, &face, &side, &direction)))
     return;
-  if (direction < 2 * MAXORIENT)
-    shift = (shift) ? 1 : 0;
-  else
-    shift = 2;
   control = (control) ? 1 : 0;
-  move_dino(w, face, side, direction, shift, control);
+  move_dino(w, face, side, direction, style, control);
   if (!control && check_solved(w)) {
     dinoCallbackStruct cb;
 
@@ -518,7 +641,7 @@ int face, side, direction, style;
     XtCallCallbacks((Widget) w, XtNselectCallback, &cb);
     corner = (side - !((side + direction) % 2)) % MAXORIENT;
     new_face = opp_corner[face][corner].face;
-    new_corner = opp_corner[face][corner].corner;
+    new_corner = opp_corner[face][corner].rotation;
     new_direction = 2 * ((direction / 2 + (face != 1 && face != 3)) % 2) +
       !(new_corner % 2);
     new_side = new_corner + 2 * MAXORIENT;
@@ -537,7 +660,7 @@ int face, side, direction, style;
     cb.style = CORNER;
     XtCallCallbacks((Widget) w, XtNselectCallback, &cb);
     corner = (side - !((side + direction) % 2)) % MAXORIENT;
-    new_corner = opp_corner[face][corner].corner;
+    new_corner = opp_corner[face][corner].rotation;
     new_face = opp_corner[face][corner].face;
     if (new_face != 1 && new_face != 3)
       new_direction = (direction + 2) % MAXORIENT;
@@ -551,6 +674,23 @@ int face, side, direction, style;
     cb.direction = new_direction;
     cb.style = CORNER;
     XtCallCallbacks((Widget) w, XtNselectCallback, &cb);
+  } else if (style == EDGE) {
+    new_side = (side + 2) % MAXORIENT; 
+    move_polyhedrons(w, face, new_side, direction, EDGE);
+    cb.face = face;
+    cb.side = new_side;
+    cb.direction = direction;
+    cb.style = EDGE;
+    XtCallCallbacks((Widget) w, XtNselectCallback, &cb);
+  }
+}
+
+static void check_polyhedrons(w)
+DinoWidget w;
+{
+  if (w->dino.mode < PERIOD2 || w->dino.mode > BOTH) {
+    XtWarning("Mode is in error, use 2 for Period2, 3 for Period3, 4 for Both");
+    w->dino.mode = DEFAULTMODE;
   }
 }
 
@@ -560,8 +700,10 @@ DinoWidget w;
   int face, side;
 
   for (face = 0; face < MAXFACES; face++)
-    for (side = 0; side < MAXORIENT; side++)
-      w->dino.cube_loc[face][side] = face;
+    for (side = 0; side < MAXORIENT; side++) {
+      w->dino.cube_loc[face][side].face = face;
+      w->dino.cube_loc[face][side].rotation = STRT - MAXORIENT;
+    }
   flush_moves(w);
   w->dino.started = FALSE;
 }
@@ -604,7 +746,12 @@ DinoWidget w;
     face = NRAND(MAXFACES);
     side = NRAND(MAXORIENT);
     direction = ((NRAND(2)) ? side + 1 : side + 3) % MAXORIENT;
-    style = NRAND(2);
+    if (w->dino.mode == PERIOD2)
+      style = EDGE;
+    else if (w->dino.mode == BOTH)
+      style = NRAND(3);
+    else
+      style = NRAND(2);
     move_dino(w, face, side, direction, style, FALSE);
   }
   flush_moves(w);
@@ -627,21 +774,27 @@ int face, side, direction, style;
     move_inside_corners(w, face, corner, direction);
   } else if (style == MIDDLE) {
     move_outside_corners(w, face, corner, direction);
-    new_corner = opp_corner[face][corner].corner;
+    new_corner = opp_corner[face][corner].rotation;
     face = opp_corner[face][corner].face;
+    if (((face != 1 && face != 3) + corner) % 2)
+      direction = (direction + 1) % MAXORIENT;
+    else
+      direction = (direction + 3) % MAXORIENT;
     corner = new_corner;
-    if (face != 1 && face != 3)
-      direction = (direction + 2) % MAXORIENT;
     move_outside_corners(w, face, corner, direction);
+  } else if (style == EDGE) {
+    move_edges(w, face, corner, direction);
+    move_faces(w, face, corner,
+      ((face == 2 || face == 5) ? CCW : HALF) % MAXORIENT);
   } else {
     int k, new_face, rotate, new_direction;
 
     rotate_face(w, face_to_rotate[face][direction % MAXORIENT], CW);
     rotate_face(w, face_to_rotate[face][(direction + 2) % MAXORIENT], CCW);
     read_face((DinoWidget) w, face, 0);
-    for (k = 1; k<= MAXORIENT; k++) {
+    for (k = 1; k <= MAXORIENT; k++) {
       new_face = slide_next_face[face][direction % MAXORIENT].face;
-      rotate = slide_next_face[face][direction % MAXORIENT].corner;
+      rotate = slide_next_face[face][direction % MAXORIENT].rotation;
       new_direction = (rotate + direction) % MAXORIENT;
       if (k != MAXORIENT)
         read_face((DinoWidget) w, new_face, k);
@@ -666,7 +819,8 @@ int face, corner, direction;
     new_direction = 2 * slide_corner[face][corner][dir].dir + !(new_corner % 2);
     if (k != MAXROTATE)
       read_diagonal((DinoWidget) w, new_face, new_corner, k);
-    write_diagonal(w, new_face, new_corner, k - 1);
+    write_diagonal(w, new_face, new_corner,
+      (new_direction - direction + MAXORIENT) % MAXORIENT, k - 1);
     face = new_face;
     corner = new_corner;
     direction = new_direction;
@@ -688,26 +842,72 @@ int face, corner, direction;
     new_direction = 2 * slide_corner[face][corner][dir].dir + !(new_corner % 2);
     if (k != MAXROTATE)
       read_diagonal((DinoWidget) w, new_face, new_corner, k);
-    write_diagonal(w, new_face, new_corner, k - 1);
+    write_diagonal(w, new_face, new_corner,
+      (new_direction - direction + MAXORIENT) % MAXORIENT, k - 1);
     face = new_face;
     corner = new_corner;
     direction = new_direction;
   }
 }
 
+static void move_edges(w, face, corner, direction)
+DinoWidget w;
+int face, corner, direction;
+{
+  int k, new_face, rotate, new_corner, new_direction;
+
+  read_diagonal((DinoWidget) w, face, corner, 0);
+  for (k = 1; k <= 2; k++) {
+    new_face = opp_face[face];
+    /*rotate = (((face == 1 || face == 3) ? 1 : 3) + 3 * direction) %
+      MAXORIENT;*/
+    new_corner = ((((face == 1 || face == 3) + corner) % 2) ?
+      (corner + 3) : (corner + 1)) % MAXORIENT;
+   rotate = (new_corner - corner + MAXORIENT) % MAXORIENT; 
+   new_direction = (rotate + direction) % MAXORIENT;
+    if (k != 2)
+      read_diagonal((DinoWidget) w, new_face, new_corner, k);
+    write_diagonal(w, new_face, new_corner, rotate, k - 1);
+    face = new_face;
+    corner = new_corner;
+    direction = new_direction;
+  }
+}
+
+static void move_faces(w, f, d, rotate)
+DinoWidget w;
+int f, d, rotate;
+{
+  int k, face, new_face;
+
+  face = face_to_rotate2[f][d][0],
+  read_face((DinoWidget) w, face, 0);
+  for (k = 1; k <= 2; k++) {
+    new_face = face_to_rotate2[f][d][k % 2],
+    rotate = MAXORIENT - rotate;
+    if (k != 2)
+      read_face((DinoWidget) w, new_face, k);
+    write_face(w, new_face, rotate, k - 1);
+    face = new_face;
+  }
+}
+
 static void read_diagonal(w, face, corner, h)
 DinoWidget w;
 int face, corner, h; 
 {
   w->dino.spindle_loc[h][0] = w->dino.cube_loc[face][corner];
   w->dino.spindle_loc[h][1] = w->dino.cube_loc[face][(corner + 1) % MAXORIENT];
-
 }
 
-static void write_diagonal(w, face, corner, h)
+static void write_diagonal(w, face, corner, rotate, h)
 DinoWidget w;
-int face, corner, h;
+int face, corner, rotate, h;
 {
+  w->dino.spindle_loc[h][0].rotation =
+    (w->dino.spindle_loc[h][0].rotation + rotate) % MAXORIENT;
+  w->dino.spindle_loc[h][1].rotation =
+    (w->dino.spindle_loc[h][1].rotation + rotate) % MAXORIENT;
   w->dino.cube_loc[face][corner] = w->dino.spindle_loc[h][0];
   draw_triangle(w, face, corner); 
   w->dino.cube_loc[face][(corner + 1) % MAXORIENT] = w->dino.spindle_loc[h][1];
@@ -728,11 +928,13 @@ static void write_face(w, face, rotate, h)
 DinoWidget w;
 int face, rotate, h;
 {
-  int side;
+  int side, new_side;
 
   for (side = 0; side < MAXORIENT; side++) {
-    w->dino.cube_loc[face][(side + rotate) % MAXORIENT] =
-      w->dino.row_loc[h][side];
+    new_side = (side + rotate) % MAXORIENT;
+    w->dino.cube_loc[face][new_side] = w->dino.row_loc[h][side];
+    w->dino.cube_loc[face][new_side].rotation =
+      (w->dino.cube_loc[face][new_side].rotation + rotate) % MAXORIENT;
     draw_triangle(w, face, (side + rotate) % MAXORIENT);
   }
 }
@@ -751,6 +953,8 @@ int face, direction;
     w->dino.cube_loc[face][side] = (direction == CW) ?
       w->dino.face_loc[(side + MAXORIENT - 1) % MAXORIENT]:
       w->dino.face_loc[(side + 1) % MAXORIENT];
+    w->dino.cube_loc[face][side].rotation =
+      (w->dino.cube_loc[face][side].rotation + direction) % MAXORIENT;
     draw_triangle(w, face, side);
   }
 }
@@ -779,13 +983,17 @@ Boolean check_solved(w)
 DinoWidget w;
 {
   int face, side;
-  int test =  -1;
+  DinoCornerLoc test;
 
   for (face = 0; face < MAXFACES; face++)
     for (side = 0; side < MAXORIENT; side++) {
-      if (side == 0)
-        test = w->dino.cube_loc[face][side];
-      else if (test != /*face*/ w->dino.cube_loc[face][side])
+      if (side == 0) {
+        test.face = w->dino.cube_loc[face][side].face;
+        test.rotation = w->dino.cube_loc[face][side].rotation;
+      } else if (test.face != /*face*/
+               w->dino.cube_loc[face][side].face ||
+               (w->dino.orient && test.rotation != /*STRT - MAXORIENT*/
+                w->dino.cube_loc[face][side].rotation))
         return FALSE;
     }
   return TRUE;
@@ -800,7 +1008,8 @@ DinoWidget w;
 
   for (face = 0; face < MAXFACES; face++) {
     for (side = 0; side < MAXORIENT; side++)
-      (void) printf("%d ", w->dino.cube_loc[face][side]);
+      (void) printf("%d %d  ", w->dino.cube_loc[face][side].face,
+               w->dino.cube_loc[face][side].rotation);
     (void) printf("\n");
   }
   (void) printf("\n");
