@@ -47,6 +47,7 @@ static void move_dino2d_bottom();
 static void move_dino2d_br();
 static void resize_polyhedrons();
 static void draw_frame();
+static void draw_orient_line();
 
 static char defaultTranslationsDino2D[] =
   "<KeyPress>q: quit()\n\
@@ -84,7 +85,11 @@ static char defaultTranslationsDino2D[] =
    <KeyPress>e: enter()\n\
    <KeyPress>w: write()\n\
    <KeyPress>u: undo()\n\
-   <KeyPress>s: solve()";
+   <KeyPress>s: solve()\n\
+   <KeyPress>o: orientize()\n\
+   <KeyPress>2: period2()\n\
+   <KeyPress>3: period3()\n\
+   <KeyPress>b: both()";
 
 static XtActionsRec actionsListDino2D[] =
 {
@@ -104,7 +109,11 @@ static XtActionsRec actionsListDino2D[] =
   {"enter", (XtActionProc) enter_dino},
   {"write", (XtActionProc) write_dino},
   {"undo", (XtActionProc) undo_dino},
-  {"solve", (XtActionProc) solve_dino}
+  {"solve", (XtActionProc) solve_dino},
+  {"orientize", (XtActionProc) orientize_dino},
+  {"period2", (XtActionProc) period2_mode_dino},
+  {"period3", (XtActionProc) period3_mode_dino},
+  {"both", (XtActionProc) both_mode_dino}
 };
 
 static XtResource resourcesDino2D[] =
@@ -127,6 +136,10 @@ static XtResource resourcesDino2D[] =
    XtOffset(DinoWidget, core.width), XtRString, "300"},
   {XtNheight, XtCHeight, XtRDimension, sizeof(Dimension),
    XtOffset(DinoWidget, core.height), XtRString, "400"},
+  {XtNmode, XtCMode, XtRInt, sizeof(int),
+   XtOffset(DinoWidget, dino.mode), XtRString, "3"}, /*DEFAULTMODE*/
+  {XtNorient, XtCOrient, XtRBoolean, sizeof(Boolean),
+   XtOffset(DinoWidget, dino.orient), XtRString, "FALSE"}, /* DEFAULTORIENT */
   {XtNmono, XtCMono, XtRBoolean, sizeof(Boolean),
    XtOffset(DinoWidget, dino.mono), XtRString, "FALSE"},
   {XtNface, XtCFace, XtRInt, sizeof(int),
@@ -275,8 +288,10 @@ char *args[];
 int n_args;
 {
   move_dino_input((DinoWidget) w, event->xbutton.x, event->xbutton.y, TL,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
-    (int) (event->xkey.state & ControlMask));
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
+    (int) (event->xkey.state & ControlMask),
+    (int) (event->xkey.state &
+      (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
 }
 
 static void move_dino2d_top(w, event, args, n_args)
@@ -286,8 +301,10 @@ char *args[];
 int n_args;
 {
   move_dino_input((DinoWidget) w, event->xbutton.x, event->xbutton.y, TOP,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
-    (int) (event->xkey.state & ControlMask));
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
+    (int) (event->xkey.state & ControlMask),
+    (int) (event->xkey.state &
+      (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
 }
 
 static void move_dino2d_tr(w, event, args, n_args)
@@ -297,8 +314,10 @@ char *args[];
 int n_args;
 {
   move_dino_input((DinoWidget) w, event->xbutton.x, event->xbutton.y, TR,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
-    (int) (event->xkey.state & ControlMask));
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
+    (int) (event->xkey.state & ControlMask),
+    (int) (event->xkey.state &
+      (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
 }
 
 static void move_dino2d_left(w, event, args, n_args)
@@ -308,8 +327,10 @@ char *args[];
 int n_args;
 {
   move_dino_input((DinoWidget) w, event->xbutton.x, event->xbutton.y, LEFT,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
-    (int) (event->xkey.state & ControlMask));
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
+    (int) (event->xkey.state & ControlMask),
+    (int) (event->xkey.state &
+      (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
 }
 
 static void move_dino2d_right(w, event, args, n_args)
@@ -319,8 +340,10 @@ char *args[];
 int n_args;
 {
   move_dino_input((DinoWidget) w, event->xbutton.x, event->xbutton.y, RIGHT,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
-    (int) (event->xkey.state & ControlMask));
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
+    (int) (event->xkey.state & ControlMask),
+    (int) (event->xkey.state &
+      (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
 }
 
 static void move_dino2d_bl(w, event, args, n_args)
@@ -330,8 +353,10 @@ char *args[];
 int n_args;
 {
   move_dino_input((DinoWidget) w, event->xbutton.x, event->xbutton.y, BL,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
-    (int) (event->xkey.state & ControlMask));
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
+    (int) (event->xkey.state & ControlMask),
+    (int) (event->xkey.state &
+      (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
 }
 
 static void move_dino2d_bottom(w, event, args, n_args)
@@ -341,8 +366,10 @@ char *args[];
 int n_args;
 {
   move_dino_input((DinoWidget) w, event->xbutton.x, event->xbutton.y, BOTTOM,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
-    (int) (event->xkey.state & ControlMask));
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
+    (int) (event->xkey.state & ControlMask),
+    (int) (event->xkey.state &
+      (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
 }
 
 static void move_dino2d_br(w, event, args, n_args)
@@ -352,8 +379,10 @@ char *args[];
 int n_args;
 {
   move_dino_input((DinoWidget) w, event->xbutton.x, event->xbutton.y, BR,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
-    (int) (event->xkey.state & ControlMask));
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
+    (int) (event->xkey.state & ControlMask),
+    (int) (event->xkey.state &
+      (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
 }
 
 static void resize_polyhedrons(w)
@@ -361,6 +390,7 @@ Dino2DWidget w;
 {
   int i, j;
 
+  w->dino.orient_line_length = w->dino2d.diamond_length / 3;
   w->dino.letter_offset.x = -2;
   w->dino.letter_offset.y = 4;
   for (i = 0; i < MAXORIENT; i++)
@@ -510,8 +540,9 @@ void draw_triangle2d(w, face, side)
 Dino2DWidget w;
 int face, side;
 {
-  int dx, dy, letter_x, letter_y, new_side;
+  int dx, dy, letter_x, letter_y, new_side, orient;
 
+  orient = w->dino.cube_loc[face][side].rotation;
   if (w->dino.vertical || face != MAXFACES - 1) {
     dx = (cube_to_plane[face] % MAXX) * w->dino2d.view_length;
     dy = (cube_to_plane[face] / MAXX) * w->dino2d.view_length;
@@ -520,22 +551,64 @@ int face, side;
     dx = (cube_to_plane[face] / MAXX) * w->dino2d.view_length;
     dy = (cube_to_plane[face] % MAXX) * w->dino2d.view_length;
     new_side = (side + HALF) % STRT;
+    orient = (orient + HALF) % STRT;
   }
   dx += w->dino.puzzle_offset.x + 2 * w->dino.delta;
   dy += w->dino.puzzle_offset.y + 2 * w->dino.delta;
-  letter_x = dx + letter_list[new_side].x;
+  letter_x = dx + letter_list[new_side].x - 1;
   letter_y = dy + letter_list[new_side].y;
   triangle_list[new_side][0].x = offset_list[new_side].x + dx;
   triangle_list[new_side][0].y = offset_list[new_side].y + dy;
   XFillPolygon(XtDisplay(w), XtWindow(w),
-    w->dino.face_GC[w->dino.cube_loc[face][side]],
+    w->dino.face_GC[w->dino.cube_loc[face][side].face],
     triangle_list[new_side], 3, Convex, CoordModePrevious);
   if (w->dino.depth == 1 || w->dino.mono) {
     char buf[2];
 
     (void) sprintf(buf, "%c",
-             w->dino.face_name[w->dino.cube_loc[face][side]][0]);
+             w->dino.face_name[w->dino.cube_loc[face][side].face][0]);
     XDrawString(XtDisplay(w), XtWindow(w), w->dino.inverse_GC,
       letter_x, letter_y, buf, 1);
   }
+  if (w->dino.orient)
+    draw_orient_line(w, orient + 2 * MAXORIENT,
+      letter_x - w->dino.letter_offset.x, letter_y - w->dino.letter_offset.y);
+}
+
+static void draw_orient_line(w, orient, dx, dy)
+Dino2DWidget w;
+int orient, dx, dy;
+{
+  switch (orient) {
+    case TOP:
+      XDrawLine(XtDisplay(w), XtWindow(w), w->dino.inverse_GC,
+                dx,
+                dy - w->dino.delta,
+                dx,
+                dy - w->dino.delta - w->dino.orient_line_length / 2);
+      return;
+    case RIGHT:
+      XDrawLine(XtDisplay(w), XtWindow(w), w->dino.inverse_GC,
+                dx + w->dino.delta,
+                dy,
+                dx + w->dino.delta + w->dino.orient_line_length / 2,
+                dy);
+      return;
+    case BOTTOM:
+      XDrawLine(XtDisplay(w), XtWindow(w), w->dino.inverse_GC,
+                dx,
+                dy + w->dino.delta,
+                dx,
+                dy + w->dino.delta + w->dino.orient_line_length / 2);
+      return;
+    case LEFT:
+      XDrawLine(XtDisplay(w), XtWindow(w), w->dino.inverse_GC,
+                dx - w->dino.delta,
+                dy,
+                dx - w->dino.delta - w->dino.orient_line_length / 2,
+                dy);
+      return;
+    default:
+      (void) printf("draw_orient_line: orient %d\n", orient);
+  }
 }
