@@ -95,7 +95,11 @@ static char defaultTranslationsDino3D[] =
    <KeyPress>e: enter()\n\
    <KeyPress>w: write()\n\
    <KeyPress>u: undo()\n\
-   <KeyPress>s: solve()";
+   <KeyPress>s: solve()\n\
+   <KeyPress>o: orientize()\n\
+   <KeyPress>2: period2()\n\
+   <KeyPress>3: period3()\n\
+   <KeyPress>b: both()";
 
 static XtActionsRec actionsListDino3D[] =
 {
@@ -117,7 +121,11 @@ static XtActionsRec actionsListDino3D[] =
   {"enter", (XtActionProc) enter_dino},
   {"write", (XtActionProc) write_dino},
   {"undo", (XtActionProc) undo_dino},
-  {"solve", (XtActionProc) solve_dino}
+  {"solve", (XtActionProc) solve_dino},
+  {"orientize", (XtActionProc) orientize_dino},
+  {"period2", (XtActionProc) period2_mode_dino},
+  {"period3", (XtActionProc) period3_mode_dino},
+  {"both", (XtActionProc) both_mode_dino}
 };
 
 static XtResource resourcesDino3D[] =
@@ -140,6 +148,10 @@ static XtResource resourcesDino3D[] =
    XtOffset(DinoWidget, core.width), XtRString, "250"},
   {XtNheight, XtCHeight, XtRDimension, sizeof(Dimension),
    XtOffset(DinoWidget, core.height), XtRString, "400"},
+  {XtNmode, XtCMode, XtRInt, sizeof(int),
+   XtOffset(DinoWidget, dino.mode), XtRString, "3"}, /*DEFAULTMODE*/
+  {XtNorient, XtCOrient, XtRBoolean, sizeof(Boolean),
+   XtOffset(DinoWidget, dino.orient), XtRString, "FALSE"}, /* DEFAULTORIENT */
   {XtNmono, XtCMono, XtRBoolean, sizeof(Boolean),
    XtOffset(DinoWidget, dino.mono), XtRString, "FALSE"},
   {XtNface, XtCFace, XtRInt, sizeof(int),
@@ -210,6 +222,7 @@ static XPoint diamond_loc_3d[MAXFACES][MAXORIENT];
 static XPoint tiny_diamond_loc_3d[MAXFACES][MAXORIENT];
 static XPoint triangle_loc_3d[MAXFACES][MAXORIENT][3];
 static XPoint letter_3d_list[MAXFACES][MAXORIENT];
+static XPoint orient_triangle[MAXFACES][MAXORIENT][2];
 
 static void InitializeDino3D(request, new)
 Widget request, new;
@@ -308,8 +321,10 @@ char *args[];
 int n_args;
 {
   move_dino_input((DinoWidget) w, event->xbutton.x, event->xbutton.y, TL,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
-    (int) (event->xkey.state & ControlMask));
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
+    (int) (event->xkey.state & ControlMask),
+    (int) (event->xkey.state &
+      (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
 }
 
 static void move_dino3d_top(w, event, args, n_args)
@@ -319,8 +334,10 @@ char *args[];
 int n_args;
 {
   move_dino_input((DinoWidget) w, event->xbutton.x, event->xbutton.y, TOP,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
-    (int) (event->xkey.state & ControlMask));
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
+    (int) (event->xkey.state & ControlMask),
+    (int) (event->xkey.state &
+      (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
 }
 
 static void move_dino3d_tr(w, event, args, n_args)
@@ -330,8 +347,10 @@ char *args[];
 int n_args;
 {
   move_dino_input((DinoWidget) w, event->xbutton.x, event->xbutton.y, TR,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
-    (int) (event->xkey.state & ControlMask));
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
+    (int) (event->xkey.state & ControlMask),
+    (int) (event->xkey.state &
+      (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
 }
 
 static void move_dino3d_left(w, event, args, n_args)
@@ -341,8 +360,10 @@ char *args[];
 int n_args;
 {
   move_dino_input((DinoWidget) w, event->xbutton.x, event->xbutton.y, LEFT,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
-    (int) (event->xkey.state & ControlMask));
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
+    (int) (event->xkey.state & ControlMask),
+    (int) (event->xkey.state &
+      (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
 }
 
 static void move_dino3d_cw(w, event, args, n_args)
@@ -352,8 +373,10 @@ char *args[];
 int n_args;
 {
   move_dino_input((DinoWidget) w, event->xbutton.x, event->xbutton.y, CW,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
-    (int) (event->xkey.state & ControlMask));
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
+    (int) (event->xkey.state & ControlMask),
+    (int) (event->xkey.state &
+      (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
 }
 
 static void move_dino3d_right(w, event, args, n_args)
@@ -363,8 +386,10 @@ char *args[];
 int n_args;
 {
   move_dino_input((DinoWidget) w, event->xbutton.x, event->xbutton.y, RIGHT,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
-    (int) (event->xkey.state & ControlMask));
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
+    (int) (event->xkey.state & ControlMask),
+    (int) (event->xkey.state &
+      (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
 }
 
 static void move_dino3d_bl(w, event, args, n_args)
@@ -374,8 +399,10 @@ char *args[];
 int n_args;
 {
   move_dino_input((DinoWidget) w, event->xbutton.x, event->xbutton.y, BL,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
-    (int) (event->xkey.state & ControlMask));
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
+    (int) (event->xkey.state & ControlMask),
+    (int) (event->xkey.state &
+      (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
 }
 
 static void move_dino3d_bottom(w, event, args, n_args)
@@ -385,8 +412,10 @@ char *args[];
 int n_args;
 {
   move_dino_input((DinoWidget) w, event->xbutton.x, event->xbutton.y, BOTTOM,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
-    (int) (event->xkey.state & ControlMask));
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
+    (int) (event->xkey.state & ControlMask),
+    (int) (event->xkey.state &
+      (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
 }
 
 static void move_dino3d_br(w, event, args, n_args)
@@ -396,8 +425,10 @@ char *args[];
 int n_args;
 {
   move_dino_input((DinoWidget) w, event->xbutton.x, event->xbutton.y, BR,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
-    (int) (event->xkey.state & ControlMask));
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
+    (int) (event->xkey.state & ControlMask),
+    (int) (event->xkey.state &
+      (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
 }
 
 static void move_dino3d_ccw(w, event, args, n_args)
@@ -407,16 +438,20 @@ char *args[];
 int n_args;
 {
   move_dino_input((DinoWidget) w, event->xbutton.x, event->xbutton.y, CCW,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
-    (int) (event->xkey.state & ControlMask));
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
+    (int) (event->xkey.state & ControlMask),
+    (int) (event->xkey.state &
+      (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
 }
 
 static void resize_polyhedrons(w)
 Dino3DWidget w;
 {
-  int face, orient;
+  int face, orient, side, corner;
   XPoint subcube_loc_3d[MAXFACES][MAXORIENT];
+  XPoint orient_cube_loc_3d[2][MAXFACES][MAXORIENT];
   XPoint subdiamond_loc_3d[MAXFACES][MAXORIENT];
+  XPoint orient_diamond_loc_3d[2][MAXFACES][MAXORIENT];
 
   w->dino.letter_offset.x = -2;
   w->dino.letter_offset.y = 3;
@@ -424,18 +459,15 @@ Dino3DWidget w;
     w->dino.puzzle_offset.x;
   w->dino3d.view_middle.y = w->dino3d.face_height +
     w->dino.puzzle_offset.y;
-  for (face = 0; face < MAXFACES; face++)
-  {
+  for (face = 0; face < MAXFACES; face++) {
     face_loc_3d[face][0].x = w->dino3d.view_middle.x;
     face_loc_3d[face][0].y = w->dino3d.view_middle.y;
-    for (orient = 1; orient < MAXORIENT; orient++)
-    {
+    for (orient = 1; orient < MAXORIENT; orient++) {
       face_loc_3d[face][orient].x = w->dino3d.face_width;
       face_loc_3d[face][orient].y = w->dino3d.face_height;
     }
   }
-  if (w->dino.vertical)
-  {
+  if (w->dino.vertical) {
     face_loc_3d[0][1].x /= -2;
     face_loc_3d[0][1].y /= -1;
     face_loc_3d[0][2].y = 0;
@@ -471,9 +503,7 @@ Dino3DWidget w;
     face_loc_3d[5][2].y = 0;
     face_loc_3d[5][3].x /= -2;
     face_loc_3d[5][3].y /= -1;
-  }
-  else
-  {
+  } else {
     face_loc_3d[0][1].x /= -1;
     face_loc_3d[0][1].y /= -2;
     face_loc_3d[0][2].y /= -2;
@@ -511,14 +541,14 @@ Dino3DWidget w;
     face_loc_3d[5][3].x = 0;
   }
 
-  for (face = 0; face < MAXFACES; face++)
-  {
+  for (face = 0; face < MAXFACES; face++) {
     cube_loc_3d[face][0].x = face_loc_3d[face][0].x;
     cube_loc_3d[face][0].y = face_loc_3d[face][0].y;
     subcube_loc_3d[face][0].x = face_loc_3d[face][0].x;
     subcube_loc_3d[face][0].y = face_loc_3d[face][0].y;
-    for (orient = 1; orient < MAXORIENT; orient++)
-    {
+    orient_cube_loc_3d[0][face][0].x = orient_cube_loc_3d[1][face][0].x = 0;
+    orient_cube_loc_3d[0][face][0].y = orient_cube_loc_3d[1][face][0].y = 0;
+    for (orient = 1; orient < MAXORIENT; orient++) {
       cube_loc_3d[face][orient].x = face_loc_3d[face][orient].x - 3 *
         w->dino.delta * face_loc_3d[face][orient].x /
         w->dino3d.face_width;
@@ -531,9 +561,26 @@ Dino3DWidget w;
       subcube_loc_3d[face][orient].y = face_loc_3d[face][orient].y -
         4 * face_loc_3d[face][orient].y * w->dino.delta /
         w->dino3d.face_height;
+      orient_cube_loc_3d[0][face][orient].x = (face_loc_3d[face][orient].x -
+        5 * face_loc_3d[face][orient].x * w->dino.delta /
+        w->dino3d.face_width) / 3;
+      orient_cube_loc_3d[0][face][orient].y = (face_loc_3d[face][orient].y -
+        5 * face_loc_3d[face][orient].y * w->dino.delta /
+        w->dino3d.face_height) / 3;
+      orient_cube_loc_3d[1][face][orient].x = (face_loc_3d[face][orient].x -
+        7 * face_loc_3d[face][orient].x * w->dino.delta /
+        w->dino3d.face_width) / 6;
+      orient_cube_loc_3d[1][face][orient].y = (face_loc_3d[face][orient].y -
+        7 * face_loc_3d[face][orient].y * w->dino.delta /
+        w->dino3d.face_height) / 6;
     }
-    for (orient = 0; orient < MAXORIENT - 1; orient++)
-    {
+
+    map_from_3d(face, 1, &corner);
+    orient_triangle[face][corner][0].x = -orient_cube_loc_3d[0][face][2].x / 2;
+    orient_triangle[face][corner][0].y = -orient_cube_loc_3d[0][face][2].y / 2;
+    orient_triangle[face][corner][1].x = -orient_cube_loc_3d[1][face][2].x / 2;
+    orient_triangle[face][corner][1].y = -orient_cube_loc_3d[1][face][2].y / 2;
+    for (orient = 0; orient < MAXORIENT - 1; orient++) {
       diamond_loc_3d[face][orient].x = (cube_loc_3d[face][orient].x +
         cube_loc_3d[face][orient + 1].x) / 2;
       diamond_loc_3d[face][orient].y = (cube_loc_3d[face][orient].y +
@@ -546,6 +593,19 @@ Dino3DWidget w;
         subdiamond_loc_3d[face][orient].x; 
       tiny_diamond_loc_3d[face][orient + 1].y = diamond_loc_3d[face][orient].y -
         subdiamond_loc_3d[face][orient].y; 
+           side = corner;
+      orient_diamond_loc_3d[0][face][orient].x =
+        (orient_cube_loc_3d[0][face][orient].x +
+        orient_cube_loc_3d[0][face][orient + 1].x) / 2;
+      orient_diamond_loc_3d[0][face][orient].y =
+        (orient_cube_loc_3d[0][face][orient].y +
+        orient_cube_loc_3d[0][face][orient + 1].y) / 2;
+      orient_diamond_loc_3d[1][face][orient].x =
+        (orient_cube_loc_3d[1][face][orient].x +
+        orient_cube_loc_3d[1][face][orient + 1].x) / 2;
+      orient_diamond_loc_3d[1][face][orient].y =
+        (orient_cube_loc_3d[1][face][orient].y +
+        orient_cube_loc_3d[1][face][orient + 1].y) / 2;
     }
     /* Its a parallelagram so take advantage of that */
     diamond_loc_3d[face][orient].x = (cube_loc_3d[face][MAXORIENT - 1].x -
@@ -560,6 +620,24 @@ Dino3DWidget w;
       subdiamond_loc_3d[face][orient].x; 
     tiny_diamond_loc_3d[face][0].y = diamond_loc_3d[face][orient].y -
       subdiamond_loc_3d[face][orient].y; 
+    orient_diamond_loc_3d[0][face][orient].x =
+      (orient_cube_loc_3d[0][face][MAXORIENT - 1].x -
+      orient_cube_loc_3d[0][face][MAXORIENT / 2].x) / 2;
+    orient_diamond_loc_3d[0][face][orient].y =
+      (orient_cube_loc_3d[0][face][MAXORIENT - 1].y -
+      orient_cube_loc_3d[0][face][MAXORIENT / 2].y) / 2;
+    orient_diamond_loc_3d[1][face][orient].x =
+      (orient_cube_loc_3d[1][face][MAXORIENT - 1].x -
+      orient_cube_loc_3d[1][face][MAXORIENT / 2].x) / 2;
+    orient_diamond_loc_3d[1][face][orient].y =
+      (orient_cube_loc_3d[1][face][MAXORIENT - 1].y -
+      orient_cube_loc_3d[1][face][MAXORIENT / 2].y) / 2;
+
+    map_from_3d(face, 1, &corner);
+    orient_triangle[face][corner][0].x = -orient_cube_loc_3d[0][face][2].x / 2;
+    orient_triangle[face][corner][0].y = -orient_cube_loc_3d[0][face][2].y / 2;
+    orient_triangle[face][corner][1].x = -orient_cube_loc_3d[1][face][2].x / 2;
+    orient_triangle[face][corner][1].y = -orient_cube_loc_3d[1][face][2].y / 2;
 
     triangle_loc_3d[face][0][1].x = subdiamond_loc_3d[face][2].x;
     triangle_loc_3d[face][0][2].x = -subdiamond_loc_3d[face][1].x -
@@ -593,6 +671,22 @@ Dino3DWidget w;
         (2 * triangle_loc_3d[face][orient][1].y +
          triangle_loc_3d[face][orient][2].y) / 3;
     }
+    for (orient = 1; orient < MAXORIENT; orient++) {
+      side = corner;
+      map_from_3d(face, (orient + 1) % MAXORIENT, &corner);
+      orient_triangle[face][corner][0].x =
+        orient_triangle[face][side][0].x +
+        orient_diamond_loc_3d[0][face][orient].x;
+      orient_triangle[face][corner][0].y =
+        orient_triangle[face][side][0].y +
+        orient_diamond_loc_3d[0][face][orient].y;
+      orient_triangle[face][corner][1].x =
+        orient_triangle[face][side][1].x +
+        orient_diamond_loc_3d[1][face][orient].x;
+      orient_triangle[face][corner][1].y =
+        orient_triangle[face][side][1].y +
+        orient_diamond_loc_3d[1][face][orient].y;
+    }
   }
   w->dino3d.cube_width = w->dino3d.face_width - 2 * w->dino.delta;
   w->dino3d.cube_height = w->dino3d.face_height - 2 * w->dino.delta;
@@ -785,17 +879,27 @@ int face, side;
   triangle_loc_3d[face][corner][0].x = dx;
   triangle_loc_3d[face][corner][0].y = dy;
   XFillPolygon(XtDisplay(w), XtWindow(w),
-    w->dino.face_GC[w->dino.cube_loc[face][side]],
+    w->dino.face_GC[w->dino.cube_loc[face][side].face],
     triangle_loc_3d[face][corner], 3, Convex, CoordModePrevious);
   if (w->dino.depth == 1 || w->dino.mono) {
     char buf[2];
 
     (void) sprintf (buf, "%c",
-      w->dino.face_name[w->dino.cube_loc[face][side]][0]);
+      w->dino.face_name[w->dino.cube_loc[face][side].face][0]);
     XDrawString(XtDisplay(w), XtWindow(w), w->dino.inverse_GC,
       letter_x + w->dino.letter_offset.x, letter_y + w->dino.letter_offset.y,
       buf, 1);
   }
+  if (w->dino.orient)
+    XDrawLine(XtDisplay(w), XtWindow(w), w->dino.inverse_GC,
+      letter_x +
+        orient_triangle[face][w->dino.cube_loc[face][side].rotation][0].x,
+      letter_y +
+        orient_triangle[face][w->dino.cube_loc[face][side].rotation][0].y,
+      letter_x +
+        orient_triangle[face][w->dino.cube_loc[face][side].rotation][1].x,
+      letter_y +
+        orient_triangle[face][w->dino.cube_loc[face][side].rotation][1].y);
 }
 
 static void map_to_3d(face, corner, side)
