@@ -579,7 +579,7 @@ char *args[];
 int n_args;
 {
   move_mlink_input(w, event->xbutton.x, TOP,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
     (int) (event->xkey.state & ControlMask));
 }
 
@@ -590,7 +590,7 @@ char *args[];
 int n_args;
 {
   move_mlink_input(w, event->xbutton.x, LEFT,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
     (int) (event->xkey.state & ControlMask));
 }
 
@@ -601,7 +601,7 @@ char *args[];
 int n_args;
 {
   move_mlink_input(w, event->xbutton.x, RIGHT,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
     (int) (event->xkey.state & ControlMask));
 }
 
@@ -612,7 +612,7 @@ char *args[];
 int n_args;
 {
   move_mlink_input(w, event->xbutton.x, BOTTOM,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
     (int) (event->xkey.state & ControlMask));
 }
 
