@@ -608,7 +608,7 @@ int n_args;
       }
     else
       (void) printf("%s corrupted: mode %d should be between %d and %d\n",
-         DATAFILE, size, PERIOD3, BOTH);
+         DATAFILE, mode, PERIOD3, BOTH);
     while ((c = getc(fp)) != EOF && c != ':');
     (void) fscanf(fp, "%d", &sticky);
     if (w->oct.sticky != (Boolean) sticky) {
@@ -796,7 +796,7 @@ char *args[];
 int n_args;
 {
   move_oct_input(w, event->xbutton.x, event->xbutton.y, CCW,
-    (int) ((event->xbutton.state & ShiftMask) | (event->xbutton.state & LockMask)),
+    (int) (event->xbutton.state & (ShiftMask | LockMask)),
     (int) (event->xbutton.state & ControlMask));
 }
 
@@ -807,7 +807,7 @@ char *args[];
 int n_args;
 {
   move_oct_input(w, event->xbutton.x, event->xbutton.y, TL,
-    (int) ((event->xbutton.state & ShiftMask) | (event->xbutton.state & LockMask)),
+    (int) (event->xbutton.state & (ShiftMask | LockMask)),
     (int) (event->xkey.state & ControlMask));
 }
 
@@ -818,7 +818,7 @@ char *args[];
 int n_args;
 {
   move_oct_input(w, event->xbutton.x, event->xbutton.y, TOP,
-    (int) ((event->xbutton.state & ShiftMask) | (event->xbutton.state & LockMask)),
+    (int) (event->xbutton.state & (ShiftMask | LockMask)),
     (int) (event->xkey.state & ControlMask));
 }
 
@@ -829,7 +829,7 @@ char *args[];
 int n_args;
 {
   move_oct_input(w, event->xbutton.x, event->xbutton.y, TR,
-    (int) ((event->xbutton.state & ShiftMask) | (event->xbutton.state & LockMask)),
+    (int) (event->xbutton.state & (ShiftMask | LockMask)),
     (int) (event->xkey.state & ControlMask));
 }
 
@@ -840,7 +840,7 @@ char *args[];
 int n_args;
 {
   move_oct_input(w, event->xbutton.x, event->xbutton.y, LEFT,
-    (int) ((event->xbutton.state & ShiftMask) | (event->xbutton.state & LockMask)),
+    (int) (event->xbutton.state & (ShiftMask | LockMask)),
     (int) (event->xkey.state & ControlMask));
 }
 
@@ -851,7 +851,7 @@ char *args[];
 int n_args;
 {
   move_oct_input(w, event->xbutton.x, event->xbutton.y, CW,
-    (int) ((event->xbutton.state & ShiftMask) | (event->xbutton.state & LockMask)),
+    (int) (event->xbutton.state & (ShiftMask | LockMask)),
     (int) (event->xkey.state & ControlMask));
 }
 
@@ -862,7 +862,7 @@ char *args[];
 int n_args;
 {
   move_oct_input(w, event->xbutton.x, event->xbutton.y, RIGHT,
-    (int) ((event->xbutton.state & ShiftMask) | (event->xbutton.state & LockMask)),
+    (int) (event->xbutton.state & (ShiftMask | LockMask)),
     (int) (event->xkey.state & ControlMask));
 }
 
@@ -873,7 +873,7 @@ char *args[];
 int n_args;
 {
   move_oct_input(w, event->xbutton.x, event->xbutton.y, BL,
-    (int) ((event->xbutton.state & ShiftMask) | (event->xbutton.state & LockMask)),
+    (int) (event->xbutton.state & (ShiftMask | LockMask)),
     (int) (event->xkey.state & ControlMask));
 }
 
@@ -884,7 +884,7 @@ char *args[];
 int n_args;
 {
   move_oct_input(w, event->xbutton.x, event->xbutton.y, BOTTOM,
-    (int) ((event->xbutton.state & ShiftMask) | (event->xbutton.state & LockMask)),
+    (int) (event->xbutton.state & (ShiftMask | LockMask)),
     (int) (event->xkey.state & ControlMask));
 }
 
@@ -895,7 +895,7 @@ char *args[];
 int n_args;
 {
   move_oct_input(w, event->xbutton.x, event->xbutton.y, BR,
-    (int) ((event->xbutton.state & ShiftMask) | (event->xbutton.state & LockMask)),
+    (int) (event->xbutton.state & (ShiftMask | LockMask)),
     (int) (event->xkey.state & ControlMask));
 }
 
