@@ -545,7 +545,7 @@ int n_args;
       }
     else
       (void) printf("%s corrupted: mode %d should be between %d and %d\n",
-         DATAFILE, size, PERIOD2, BOTH);
+         DATAFILE, mode, PERIOD2, BOTH);
     while ((c = getc(fp)) != EOF && c != ':');
     (void) fscanf(fp, "%d", &sticky);
     if (w->pyraminx.sticky != (Boolean) sticky) {
@@ -730,7 +730,7 @@ char *args[];
 int n_args;
 {
   move_pyraminx_input(w, event->xbutton.x, event->xbutton.y, CCW,
-    (int) ((event->xbutton.state & ShiftMask) | (event->xbutton.state & LockMask)),
+    (int) (event->xbutton.state & (ShiftMask | LockMask)),
     (int) (event->xbutton.state & ControlMask));
 }
 
@@ -741,7 +741,7 @@ char *args[];
 int n_args;
 {
   move_pyraminx_input(w, event->xbutton.x, event->xbutton.y, TOP,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
     (int) (event->xkey.state & ControlMask));
 }
 
@@ -752,7 +752,7 @@ char *args[];
 int n_args;
 {
   move_pyraminx_input(w, event->xbutton.x, event->xbutton.y, TR,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
     (int) (event->xkey.state & ControlMask));
 }
 
@@ -763,7 +763,7 @@ char *args[];
 int n_args;
 {
   move_pyraminx_input(w, event->xbutton.x, event->xbutton.y, LEFT,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
     (int) (event->xkey.state & ControlMask));
 }
 
@@ -774,7 +774,7 @@ char *args[];
 int n_args;
 {
   move_pyraminx_input(w, event->xbutton.x, event->xbutton.y, CW,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
     (int) (event->xkey.state & ControlMask));
 }
 
@@ -785,7 +785,7 @@ char *args[];
 int n_args;
 {
   move_pyraminx_input(w, event->xbutton.x, event->xbutton.y, RIGHT,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
     (int) (event->xkey.state & ControlMask));
 }
 
@@ -796,7 +796,7 @@ char *args[];
 int n_args;
 {
   move_pyraminx_input(w, event->xbutton.x, event->xbutton.y, BL,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
     (int) (event->xkey.state & ControlMask));
 }
 
@@ -807,7 +807,7 @@ char *args[];
 int n_args;
 {
   move_pyraminx_input(w, event->xbutton.x, event->xbutton.y, BOTTOM,
-    (int) ((event->xkey.state & ShiftMask) | (event->xkey.state & LockMask)),
+    (int) (event->xkey.state & (ShiftMask | LockMask)),
     (int) (event->xkey.state & ControlMask));
 }
 
