@@ -817,10 +817,8 @@ SkewbWidget w;
   for (face = 0; face < MAXFACES; face++)
     for (cube = 0; cube < MAXCUBES; cube++) {
       if (cube == 0) {
-        test.face =
-          w->skewb.cube_loc[face][cube].face;
-        test.rotation =
-          w->skewb.cube_loc[face][cube].rotation;
+        test.face = w->skewb.cube_loc[face][cube].face;
+        test.rotation = w->skewb.cube_loc[face][cube].rotation;
       } else if (test.face != /*face*/
                w->skewb.cube_loc[face][cube].face ||
                (w->skewb.orient && test.rotation != /*STRT - MAXORIENT*/
