@@ -397,18 +397,15 @@ Skewb3DWidget w;
     w->skewb.puzzle_offset.x;
   w->skewb3d.view_middle.y = w->skewb3d.face_height +
     w->skewb.puzzle_offset.y;
-  for (face = 0; face < MAXFACES; face++)
-  {
+  for (face = 0; face < MAXFACES; face++) {
     face_loc_3d[face][0].x = w->skewb3d.view_middle.x;
     face_loc_3d[face][0].y = w->skewb3d.view_middle.y;
-    for (orient = 1; orient < MAXORIENT; orient++)
-    {
+    for (orient = 1; orient < MAXORIENT; orient++) {
       face_loc_3d[face][orient].x = w->skewb3d.face_width;
       face_loc_3d[face][orient].y = w->skewb3d.face_height;
     }
   }
-  if (w->skewb.vertical)
-  {
+  if (w->skewb.vertical) {
     face_loc_3d[0][1].x /= -2;
     face_loc_3d[0][1].y /= -1;
     face_loc_3d[0][2].y = 0;
@@ -444,9 +441,7 @@ Skewb3DWidget w;
     face_loc_3d[5][2].y = 0;
     face_loc_3d[5][3].x /= -2;
     face_loc_3d[5][3].y /= -1;
-  }
-  else
-  {
+  } else {
     face_loc_3d[0][1].x /= -1;
     face_loc_3d[0][1].y /= -2;
     face_loc_3d[0][2].y /= -2;
@@ -484,16 +479,14 @@ Skewb3DWidget w;
     face_loc_3d[5][3].x = 0;
   }
 
-  for (face = 0; face < MAXFACES; face++)
-  {
+  for (face = 0; face < MAXFACES; face++) {
     cube_loc_3d[face][0].x = face_loc_3d[face][0].x;
     cube_loc_3d[face][0].y = face_loc_3d[face][0].y;
     subcube_loc_3d[face][0].x = face_loc_3d[face][0].x;
     subcube_loc_3d[face][0].y = face_loc_3d[face][0].y;
     orient_cube_loc_3d[0][face][0].x = orient_cube_loc_3d[1][face][0].x = 0;
     orient_cube_loc_3d[0][face][0].y = orient_cube_loc_3d[1][face][0].y = 0;
-    for (orient = 1; orient < MAXORIENT; orient++)
-    {
+    for (orient = 1; orient < MAXORIENT; orient++) {
       cube_loc_3d[face][orient].x = face_loc_3d[face][orient].x - 3 *
         w->skewb.delta * face_loc_3d[face][orient].x /
         w->skewb3d.face_width;
@@ -543,8 +536,7 @@ Skewb3DWidget w;
     triangle_loc_3d[face][3][1].y = -subcube_loc_3d[face][2].y;
     triangle_loc_3d[face][3][2].y = subcube_loc_3d[face][1].y +
       subcube_loc_3d[face][2].y;
-    for (orient = 0; orient < MAXORIENT; orient++)
-    {
+    for (orient = 0; orient < MAXORIENT; orient++) {
       letter_3d_list[face][orient].x =
         (2 * triangle_loc_3d[face][orient][1].x +
          triangle_loc_3d[face][orient][2].x) / 3;
@@ -558,8 +550,7 @@ Skewb3DWidget w;
   w->skewb3d.cube_diagonal = w->skewb3d.face_diagonal - 2 * w->skewb.delta;
   w->skewb3d.cube_diag = w->skewb3d.face_diagonal + 2 * w->skewb.delta;
 
-  if (w->skewb.vertical)
-  {
+  if (w->skewb.vertical) {
     letter_3d_list[0][MAXORIENT].x = w->skewb3d.cube_width / 4;
     letter_3d_list[0][MAXORIENT].y = -w->skewb3d.cube_height / 2 + 2;
     letter_3d_list[1][MAXORIENT].x = -w->skewb3d.cube_diagonal;
@@ -572,9 +563,7 @@ Skewb3DWidget w;
     letter_3d_list[4][MAXORIENT].y = -w->skewb3d.cube_height / 2 + 2;
     letter_3d_list[5][MAXORIENT].x = -w->skewb3d.cube_width / 4;
     letter_3d_list[5][MAXORIENT].y = w->skewb3d.cube_height / 2 - 2;
-  }
-  else
-  {
+  } else {
     letter_3d_list[0][MAXORIENT].x = 0;
     letter_3d_list[0][MAXORIENT].y = -w->skewb3d.cube_diagonal;
     letter_3d_list[1][MAXORIENT].x = -w->skewb3d.cube_width / 2 + 2;
@@ -589,10 +578,8 @@ Skewb3DWidget w;
     letter_3d_list[5][MAXORIENT].y = -w->skewb3d.cube_height / 4;
   }
 
-  for (face = 0; face < MAXFACES; face++)
-  {
-    for (orient = 0; orient < MAXORIENT - 1; orient++)
-    {
+  for (face = 0; face < MAXFACES; face++) {
+    for (orient = 0; orient < MAXORIENT - 1; orient++) {
       diamond_loc_3d[face][orient].x = (cube_loc_3d[face][orient].x +
         cube_loc_3d[face][orient + 1].x) / 2;
       diamond_loc_3d[face][orient].y = (cube_loc_3d[face][orient].y +
@@ -647,8 +634,7 @@ Skewb3DWidget w;
     orient_triangle[face][corner][0].y = -orient_cube_loc_3d[0][face][2].y / 2;
     orient_triangle[face][corner][1].x = -orient_cube_loc_3d[1][face][2].x / 2;
     orient_triangle[face][corner][1].y = -orient_cube_loc_3d[1][face][2].y / 2;
-    for (orient = 1; orient < MAXORIENT; orient++)
-    {
+    for (orient = 1; orient < MAXORIENT; orient++) {
       side = corner;
       map_from_3d(face, (orient + 1) % MAXORIENT, &corner);
       orient_diamond[face][corner][0].x =
